// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KDEGAMES_VERSION_H
#define KDEGAMES_VERSION_H

#define KDEGAMES_VERSION_STRING "7.3.0"
#define KDEGAMES_VERSION_MAJOR 7
#define KDEGAMES_VERSION_MINOR 3
#define KDEGAMES_VERSION_PATCH 0
#define KDEGAMES_VERSION ((7<<16)|(3<<8)|(0))

#endif
